// Copyright (C) 2009 by Florent Lamiraux, Thomas Moulard, CNRS.
//
// This file is part of the kpp-interfacewalk.
//
// kpp-interfacewalk is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or (at your option) any later version.
//
// kpp-interfacewalk is distributed in the hope that it will be
// useful, but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with kpp-interfacewalk.  If not, see
// <http://www.gnu.org/licenses/>.

#ifndef KPP_INTERFACE_WALK_INTERFACE_HH
# define KPP_INTERFACE_WALK_INTERFACE_HH
# include <kpp/interfacewalk/fwd.hh>

# include <kpp/interfacewalk/kppinterface-nowarning.hh>

namespace kpp
{
  namespace interfaceWalk
  {

    class Interface : public CkppInterface
    {
    public:
      typedef hpp::walkPlanner::PlannerConstShPtr PlannerConstShPtr;
      typedef hpp::walkPlanner::PlannerShPtr PlannerShPtr;

      virtual ~Interface ();

      static InterfaceShPtr create ();

      virtual std::string name () const;

    protected:
      Interface (PlannerShPtr inHppPlanner);

      /// \brief Initialization of the object
      ///
      /// Each object stores a weak pointer to itself, instatiated by
      /// calling this function.
      ktStatus init (const InterfaceWkPtr& inWeakPtr);

      /// \name Commands list.
      /// \{

      /// \brief Command to solve a planner problem.
      CkppUICommandShPtr attCommandSolvePlanner;
      /// \brief Command to set a configuration.
      CkppUICommandShPtr attCommandSetConfig;
      /// \brief Command to output the motion.
      CkppUICommandShPtr attCommandOutputMotion;
      /// \brief Command to load obstacle by obstacle server.
      CkppUICommandShPtr attCommandLoadObstacle;
      /// \brief Command to load HRP2.
      CkppUICommandShPtr attCommandLoadHRP2;

      /// Planner associed to the interface
      PlannerShPtr attHppWalkPlanner;

      /// \}
    private:
      /// \brief Weak pointer to itself
      InterfaceWkPtr attWeakPtr;
    };

  } // end of namespace interfaceWalk.
} // end of namespace kpp.


/// \brief KPP initialization function.
///
/// First command ran by Kineo when loading the module and checking
/// for the license.
extern "C" KPP_ADDON_API int
initializeModule (CkppModuleInterfaceShPtr& o_moduleInterface);

#endif //! KPP_INTERFACE_WALK_INTERFACE_HH
