// Copyright (C) 2009 by Florent Lamiraux, Thomas Moulard, CNRS.
//
// This file is part of the kpp-interfacewalk.
//
// kpp-interfacewalk is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or (at your option) any later version.
//
// kpp-interfacewalk is distributed in the hope that it will be
// useful, but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with kpp-interfacewalk.  If not, see
// <http://www.gnu.org/licenses/>.

#include <fstream>

#include "config.h"

#include <hpp/util/debug.hh>
#include <hpp/walkplanner/planner.hh>

#include "kpp/interfacewalk/interface.hh"

namespace kpp
{
  namespace interfaceWalk
  {
    Interface::Interface (PlannerShPtr planner)
      : CkppInterface (&*planner),
	attHppWalkPlanner (planner)
    {}

    Interface::~Interface ()
    {}

    InterfaceShPtr
    Interface::create ()
    {
      std::string pathToHrp2 (OPENHRP_PREFIX);
      pathToHrp2 += "/Controller/IOserver/robot/HRP2JRL/model/HRP2JRLmain.wrl";

      // Create an walkPlanner::Planner object.
      hppDout (info, "HRP-2 path: " << pathToHrp2);

      std::ifstream hrp2Wrl;
      hrp2Wrl.open (pathToHrp2.c_str (), std::ios_base::in);
      hrp2Wrl.close ();
      if (hrp2Wrl.fail ())
	{
	  hppDout (error,
		   "failed to find HRP2 VRML file at location ``"
		   << pathToHrp2.c_str () << "''." << std::endl
		   << "This value is defined at compile-time by hpp-openhrp."
		   << std::endl <<
		   "Please check that this package has been configured with "
		   "the correct --with-openhrp flag and that HRP2 VRML file "
		   "has not been moved.");
	  return InterfaceShPtr ();
	}

      PlannerShPtr planner
	(new ChppWalkPlanner(0.2, 0.2, 0.2, 0.04));

      if (!planner)
	{
	  hppDout (error, "failed to create the planner");
	  return InterfaceShPtr ();
	}

      InterfaceShPtr ptr (new Interface (planner));
      InterfaceWkPtr wkPtr (ptr);

      if (ptr->init (wkPtr) != KD_OK)
	ptr.reset ();
      return ptr;
    }

    ktStatus
    Interface::init (const InterfaceWkPtr& weakPtr)
    {
      if (CkppInterface::init (weakPtr) != KD_OK)
	return KD_ERROR;
      attWeakPtr = weakPtr;
      return KD_OK;
    }

    std::string
    Interface::name () const
    {
      return PACKAGE_STRING;
    }

  } // end of namespace interfaceWalk.
} // end of namespace kpp.

extern "C"
int
initializeModule (CkppModuleInterfaceShPtr& o_moduleInterface)
{
  using namespace kpp::interfaceWalk;
  hppDout (notice, "initializing module " << PACKAGE_STRING);
  o_moduleInterface = Interface::create ();

  if (o_moduleInterface)
    return 0;

  //FIXME: for some reason, a segv happens if the module initialization
  // fails. As the user is doomed anyway, let's exit with a nice error
  // message.
  hppDoutFatal (error, "failed to initialize the module, exiting...");
  return 1;
}
